package com.idtechproducts.usbhid.securemag.demo;

import com.idtechproducts.usbhid.sdk.Common;
import com.idtechproducts.usbhid.sdk.IDTechUsbHid;
import com.idtechproducts.usbhid.sdk.IDTechUsbHidMsg;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

/*
 * 
 * File name: 	SecureMagUsbHidDemo.java
 * Author:		Jimmy Mo
 * Date:		2014.05.15
 * 
 * Last Modified
 *  Author: Jimmy Mo
 *  Date: 2014.06.03
 * 
 */

public class SecureMagUsbHidDemo extends ActionBarActivity{

	private SecureMagSdkFragment mainView = null;
	private ProductInfoFragment productInfoView = null;
	private final static String DEMO_LOG = "SecureMag USB-HID Demo"; 
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_demo);

		if (savedInstanceState == null) {
			mainView = new SecureMagSdkFragment();
			getSupportFragmentManager().beginTransaction()
					.add(R.id.container, mainView).commit();
		}
    	getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

	}

	// Inflate the menu items to the action bar
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
		
	// Handles action bar item clicks
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch(item.getItemId()) {

		case R.id.action_switch_hid:
			mainView.switchToHID();
			break;

		case R.id.action_switch_kb:
			mainView.switchToKB();
			break;
			
		case R.id.action_product_info:
			if (productInfoView == null) {
				productInfoView = new ProductInfoFragment();
				productInfoView.init(getString(R.string.product_url));
			}
			if (productInfoView.isVisible())
				break;
			
			FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
			transaction.replace(R.id.container, productInfoView);
            transaction.addToBackStack(null);
            transaction.commit();
			return true;

		case R.id.action_sdk_info:
			mainView.displaySdkInfo();
			break;
			
		case R.id.action_enable_log:

			if (item.isChecked()) {
				item.setChecked(false);
				mainView.setEnableLog(false);
			}
			else {
				item.setChecked(true);
				mainView.setEnableLog(true);
			}
			
			break;
		case R.id.action_delete_log:
			int deleteCount = mainView.deleteLogs();
			Toast.makeText(this, deleteCount + " file(s) deleted.", Toast.LENGTH_SHORT).show();
			break;
			
		case R.id.action_exit_app:
			finish();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}


	/**
	 * SecureMag USB-HID SDK fragment
	 */
	public static class SecureMagSdkFragment extends Fragment  implements IDTechUsbHidMsg{

		private TextView status;
		private TextView infoText;
		private EditText dataText;
		private IDTechUsbHid secureMag;
		private View rootView;
		private LayoutInflater layoutInflater;
		private ViewGroup viewGroup;
		private AlertDialog alertSwipe;
		private Handler handler = new Handler();
		private String info = "";
		private String detail = "";
		
//		private ReaderType typeSelected = ReaderType.SecureMag;
		
		private String[] commands = {
				"Get Version",  		// 0 
				"Get Serial Number", 	// 1
				"Get Setting", 			// 2
				"Enable TDES", 			// 3
				"Enable AES", 			// 4
				"Get Next KSN",			// 5
				"Default Settings",		// 6 
				"Enable Error Notif.",	// 7 
				"Disable Error Notif.",	// 8 
				"Enable Exp. Date",		// 9
				"Disable Exp. Date"		// 10
		};
		Button swipeButton;// = (Button)rootView.findViewById(R.id.btn_swipeCard);
		
		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
			layoutInflater = inflater;
			viewGroup = container;
			rootView = inflater.inflate(R.layout.fragment_main, container, false);
			
			status = (TextView)rootView.findViewById(R.id.status_text);
			status.setText("Disconnected");
			
			infoText = (TextView)rootView.findViewById(R.id.text_area_top);
			dataText = (EditText)rootView.findViewById(R.id.text_area_bottom);
			dataText.setVerticalScrollBarEnabled(true);
			
			swipeButton = (Button)rootView.findViewById(R.id.btn_swipeCard);
			swipeButton.setOnClickListener(new SwipeButtonListener());
			swipeButton.setEnabled(false);
			
			Button commandBtn = (Button)rootView.findViewById(R.id.btn_command);
			commandBtn.setOnClickListener(new CommandButtonListener());
			
			return rootView;
		}
		
		// Initialize SDK after View and Activity created
		// Then registerListen() to start monitor USB devices plugged in/out from the Android device
		@Override
		public void onActivityCreated(Bundle savedInstanceState) {
//			openReaderSelectDialog();
			displaySdkInfo();
			secureMag = new IDTechUsbHid(SecureMagSdkFragment.this, getActivity());
			secureMag.registerListen();
			super.onActivityCreated(savedInstanceState);
		}
		
		void switchToHID() {
			if (secureMag == null){
				Toast.makeText(getActivity(), "SDK not initialized.", Toast.LENGTH_LONG).show();
				return;
			}
//			if (typeSelected == ReaderType.SecureMag) {
//				Toast.makeText(getActivity(), "USB-HID already selected.", Toast.LENGTH_LONG).show();
//				return;
//			}
			if (!secureMag.isReaderConnected()) {
				Toast.makeText(getActivity(), "Please connect USB-KB reader first.", Toast.LENGTH_LONG).show();
				return;
			}
			
			if (secureMag.switchToHidMode()) {
				info = 	"USB-HID mode will be enabled in few seconds.";
				detail = "";
				handler.post(doUpdateStatus);
			}
			else
				Toast.makeText(getActivity(), "Failed to switch to USB-HID mode", Toast.LENGTH_LONG).show();

		}
		
		void switchToKB() {
			if (secureMag == null){
				Toast.makeText(getActivity(), "SDK not initialized.", Toast.LENGTH_LONG).show();
				return;
			}
//			if (typeSelected == ReaderType.SecureMag_KB) {
//				Toast.makeText(getActivity(), "USB-KB already selected.", Toast.LENGTH_LONG).show();
//				return;
//			}
			if (!secureMag.isReaderConnected()) {
				Toast.makeText(getActivity(), "Please connect USB-HID reader first.", Toast.LENGTH_LONG).show();
				return;
			}
			
			if (secureMag.switchToKeyboardMode()) {
				info = 	"USB-KB mode will be enabled in few seconds.";
				detail = "";
				handler.post(doUpdateStatus);
			}
			else
				Toast.makeText(getActivity(), "Failed to switch to USB-HID mode", Toast.LENGTH_LONG).show();

		}
/*		
	    void openReaderSelectDialog() {
	    	AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
	    	builder.setTitle("Select a device:");
	    	builder.setCancelable(false);
	    	builder.setItems(R.array.reader_type, new DialogInterface.OnClickListener() {
				
				public void onClick(DialogInterface dialog, int which) {
					
					switch(which) {

					case 0:
						if (secureMag != null)
							secureMag.unregisterListen();
						typeSelected = ReaderType.SecureMag_KB;
						secureMag = new IDTechUsbHid(SecureMagSdkFragment.this, getActivity(), typeSelected);
						secureMag.registerListen();

						Toast.makeText(getActivity(), "USB-KB is selected", Toast.LENGTH_SHORT).show();
						break;
					case 1:
						if (secureMag != null)
							secureMag.unregisterListen();
						typeSelected = ReaderType.SecureMag;
						secureMag = new IDTechUsbHid(SecureMagSdkFragment.this, getActivity(), typeSelected);
						secureMag.registerListen();

						Toast.makeText(getActivity(), "USB-HID is selected", Toast.LENGTH_SHORT).show();
						break;

					}
				}
			});
	    	builder.create().show();
	    }
*/
		public void displaySdkInfo() {
			
			info = 	"Manufacturer: " + android.os.Build.MANUFACTURER + "\n" + 
					"Model: " + android.os.Build.MODEL + "\n" + 
					"OS Version: " + android.os.Build.VERSION.RELEASE + " \n" +
					"SDK Version: \n" + IDTechUsbHid.getSDKVersionInfo() + "\n";
			
			detail = "";
			
			handler.post(doUpdateStatus);
		}
		
		public void setEnableLog(boolean enableLog){
			if (secureMag == null)
				return;
			secureMag.setSaveLogEnable(enableLog);
		}
		
		public int deleteLogs() {
			if (secureMag != null)
				return secureMag.deleteLogs();
			else
				return 0;
		}

		@Override
		public void onDestroy() {
			
			if (secureMag != null)
				secureMag.unregisterListen();
			
			super.onDestroy();
		}

		@Override
		public void onReceiveMsgConnected() {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgConnected()");
			status.setText("Connected");

//			displaySdkInfo();
			if (secureMag != null)
				info = secureMag.getReaderType()+" is connected";
			else 
				info = "Reader is connected";
			detail = "Note:\nIf USB-KB device is connected, MSR data won't be displayed.\nPlease switch to USB-HID mode.";
			handler.post(doUpdateStatus);
			Toast.makeText(getActivity(), "Reader Connected", Toast.LENGTH_LONG).show();
		}

		@Override
		public void onReceiveMsgDisconnected() {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgDisconnected()");
			if (alertSwipe != null)
				if (alertSwipe.isShowing())
					alertSwipe.dismiss();
			status.setText("Disconnected");		
			info = "Reader disconnected";
			detail = "";
			handler.post(doUpdateStatus);
		}

		@Override
		public void onReceiveMsgCommandResult(int commandID, byte[] cmdReturn) {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgCommandResult()");
			if (cmdReturn == null || cmdReturn.length <= 0) {
				info = "Command failed";
				detail = "";
				handler.post(doUpdateStatus);
				return;
			}
			
			String responseString = new String(cmdReturn);
			
			switch(commandID) {
			case IDTechUsbHidMsg.cmdSwitchToHidMode:
				if (cmdReturn[0] == 0x06) {
					Log.i(DEMO_LOG, "Changed to HID mode successfully");
				}
				else 
					info = "Failed to switch to USB-HID mode";
				break;
			case IDTechUsbHidMsg.cmdSwitchToKeyboardMode:
				if (cmdReturn[0] == 0x06) {
					Log.i(DEMO_LOG, "Changed to KB mode successfully");
				}
				else 
					info = "Failed to switch to USB-KB mode";
				break;
			case IDTechUsbHidMsg.cmdGetVersion:
				info = "Firmware Version: \n" + responseString;
				break;
			case IDTechUsbHidMsg.cmdGetSerialNumber:
				info = "Serial Number: \n" + responseString;
				break;
			case IDTechUsbHidMsg.cmdGetSettings:
				info = "Firmware Settings:";
				break;
			case IDTechUsbHidMsg.cmdEnableTDES:
				if (cmdReturn[0] == 0x06)
					info = "Enable TDES: Success";
				else 
					info = "Failed to enable TDES";
				break;
			case IDTechUsbHidMsg.cmdEnableAES:
				if (cmdReturn[0] == 0x06)
					info = "Enable AES: Success";
				else 
					info = "Failed to enable AES";
				break;
			case IDTechUsbHidMsg.cmdGetNextKSN:
				info = "Next KSN:";
				break;
			case IDTechUsbHidMsg.cmdDefaultGeneralSetting:
				if (cmdReturn[0] == 0x06)
					info = "Set Default Setting: Success";
				else 
					info = "Failed to set Default Setting";
				break;
			case IDTechUsbHidMsg.cmdEnableErrorNotification:
				if (cmdReturn[0] == 0x06)
					info = "Enable Error Notification: Success";
				else 
					info = "Failed to enable Error Notification";
				break;
			case IDTechUsbHidMsg.cmdDisableErrorNotification:
				if (cmdReturn[0] == 0x06)
					info = "Disable Error Notification: Success";
				else 
					info = "Failed to disable Error Notification";
				break;
			case IDTechUsbHidMsg.cmdEnableExpDate:
				if (cmdReturn[0] == 0x06)
					info = "Enable Exp. Date: Success";
				else 
					info = "Failed to disable Exp. Date";
				break;
			case IDTechUsbHidMsg.cmdDisableExpDate:
				break;
			}
			detail = Common.getHexStringFromBytes(cmdReturn, cmdReturn.length);
			handler.post(doUpdateStatus);
		}

		@Override
		public void onReceiveMsgCommandTimeout() {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgcommandTimeout()");
			info = "Timeout on command...";
			detail = "";
			handler.post(doUpdateStatus);
		}

		@Override
		public void onReceiveMsgToSwipeCard() {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgToSwipeCard()");
			info = "Waiting for card data...";
			detail = "";

			handler.post(doSwipeProgressBar);
			handler.post(doUpdateStatus);
		}
		
		@Override
		public void onReceiveMsgCardData(byte[] cardData) {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgCardData()");
			if (alertSwipe != null)
				alertSwipe.dismiss();

			info = new String(cardData);
			detail = Common.getHexStringFromBytes(cardData, cardData.length);
//			handler.post(doUpdateStatusThenSwipe);
			handler.post(doUpdateStatus);
		
		}

		@Deprecated
		@Override
		public void onReceiveMsgSwipeTimeout() {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgSwipeTimeout()");
			if (alertSwipe != null && alertSwipe.isShowing())
				alertSwipe.dismiss();
			info = "Swipe Timeout";
			detail = "";
			handler.post(doUpdateStatus);
		}
		
		@Override
		public void onReceiveMsgFailureInfo(int errorCode) {
			Log.i(DEMO_LOG, "Callback received: onReceiveMsgFailureInfo()");
			switch(errorCode) {
			
			case ERROR_PERMISSION_DENIED:
				info = "Error; Access permission is denied.";
				break;
			case ERROR_DATA_FORMAT:
				info = "Error; Detected incorrect data format.";
				break;
			case ERROR_CARD_DATA:
				info = "Error; Detected incorrect card data.";
				break;
			default:
				info = "Error code: "+errorCode;
			}
			detail = "";
			handler.post(doUpdateStatus);
			
		}
		
		private Runnable doUpdateStatus = new Runnable()
		{
			public void run()
			{		
				infoText.setText(info);
				dataText.setText(detail);
			}
		};
		
		private Runnable doUpdateStatusThenSwipe = new Runnable()
		{
			public void run()
			{		
				infoText.setText(info);
				dataText.setText(detail);
				secureMag.startSwipeCard();
			}
		};
		
//		public boolean enableSwipe = false;
		
		private class SwipeButtonListener implements OnClickListener {
			
			@Override
			public void onClick(View arg0) {
				if (secureMag == null)
					return;

//				if (enableSwipe) {
//					enableSwipe = false;
//					secureMag.stopSwipeCard();
//					return;
//				}
//				enableSwipe = true;
				if (secureMag.isReaderConnected()) {
					secureMag.startSwipeCard();
				}
			}			
		}
		
		private class CommandButtonListener implements OnClickListener {

			@Override
			public void onClick(View arg0) {
				final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(
	                    getActivity(), android.R.layout.select_dialog_singlechoice, commands);
				AlertDialog.Builder commandBuilder = new AlertDialog.Builder(getActivity());
				commandBuilder.setTitle("Select a Command");
				commandBuilder.setAdapter(arrayAdapter, new DialogInterface.OnClickListener () {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						if (secureMag == null)
							return;
						if (!secureMag.isReaderConnected()) {
							info = "Connect a reader first";
							detail = "";
							handler.post(doUpdateStatus);
							return;
						}

						if (which <commands.length) {
							info = "To Send " + commands[which] + " Command ...";
							detail = "";
							handler.post(doUpdateStatus);
						}
						
						switch (which) {
						case 0:
							if (!secureMag.sendCommandGetVersion()){
								info = "Failed to send GetVersion command";
							}										
							break;
						case 1:
							if (!secureMag.sendCommandGetSerialNumber()){
								info = "Failed to send GetSerialNumber command";
							}
							break;
						case 2:
							if (!secureMag.sendCommandGetSettings()){
								info = "Failed to send GetSettings command";
							}
							break;
						case 3:									
							if (!secureMag.sendCommandEnableTDES()){
								info = "Failed to send EnableTDES command";
							}
							break;
						case 4:
							if (!secureMag.sendCommandEnableAES()){
								info = "Failed to send EnableAES command";
							}
							break;
						case 5:
							if (!secureMag.sendCommandGetNextKSN()){
								info = "Failed to send GetNextKSN command";
							}
							break;
						case 6:
							if (!secureMag.sendCommandDefaultGeneralSettings()){
								info = "Failed to send DefaultGeneralSettings command";
							}
							break;
						case 7:
							if (!secureMag.sendCommandEnableErrNotification()){
								info = "Failed to send EnableErrorNotification command";
							}
							break;
						case 8:
							if (!secureMag.sendCommandDisableErrNotification()){
								info = "Failed to send DisableErrorNotification command";
							}
							break;
						case 9:
							if (!secureMag.sendCommandEnableExpDate()){
								info = "Failed to send EnableExpDate command";
							}
							break;
						case 10:
							if (!secureMag.sendCommandDisableExpDate()){
								info = "Failed to send DisableExpDate command";
							}
							break;
						}
						
					}
				});
				
				commandBuilder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
					}
				});
				commandBuilder.create().show();
			}
			
		}
		
		private Runnable doSwipeProgressBar = new Runnable() {

			@Override
			public void run() {
				AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
				builder.setTitle("Please swipe a card");
				builder.setView(layoutInflater.inflate(R.layout.frame_swipe, viewGroup, false));
				builder.setCancelable(false);
				builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						infoText.setText("Swipe cancelled...");
						secureMag.stopSwipeCard();
					}
				});
				alertSwipe = builder.create();
				alertSwipe.show();
			}
			
		};
		
	}
	
	public static class ProductInfoFragment extends Fragment {
		
		private String currentURL;

		public ProductInfoFragment() {
		}
		public void init(String url){
			currentURL = url;
		}
		
		@Override
		public void onActivityCreated(Bundle savedInstanceState){
			super.onActivityCreated(savedInstanceState);
		}
		
		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
			View rootView = inflater.inflate(R.layout.fragment_info, container, false);
			
			if(currentURL != null) {
				WebView wv = (WebView) rootView.findViewById(R.id.webPage);
				wv.getSettings().setBuiltInZoomControls(true);
                wv.setWebViewClient(new WebClient());
                wv.loadUrl(currentURL);
			}
			
			return rootView;
		}
		
        private class WebClient extends WebViewClient {
            @Override
            public boolean shouldOverrideUrlLoading(WebView view, String url) {
                return false;
            }
        }
		
	}
}
